//************************************************************************************
// KS-GARDEN (Plant pouring, growlight and fan system for 4 plants)
//************************************************************************************
// BASE
// 1 RTC (Real Time Clock) -> For time dependent light switching and to start
//                            the measurements every hour
// 1 Arduino -> The "brain" of the KS-GARDEN
// 1 OLED Display 128x64
//   Shows day of month, hour, minute and measured values of the last measurement of
//   all four sensors. (LINE 1. to 4. - Numbered 1. to 4.)
//   Shows day of month, hour, minute and measured values of the last time the plants
//   where poured for all four sensors/plants. (LINE 5. to 8. - Numbered 1. to 4.)
//   Line example:
//   1.18-16:00-234
//   First Sensor, Day 18 of the Month at 16:00 measured 234
//   (The first line shows the actual temperature in the grow box, the second
//    line shows the actual humidity in the grow box)
// 
// LIGHT
// 1 Power supply 230VAC to 12VDC
//   ->For Grow Light-LED-Cooling-Ventilators
// 1 Relais Module (HIGH = lights on, LOW = lights off)
//   ->To turn on the 230VAC for the LED's and the 12VDC for the coolers
// 6 50 Watt Leds 230VAC with cooler ventilators 12VDC
//
// HUMIDITY AND TEMPERATURE
// 1 DHT11
//   ->Temperature and humidity sensor
// 1 Relais MOdule (HIGH = fan on, LOW = fan off)
//   ->To switch the fan on/off
//
// WATER
// 4 Relais Module (HIGH = Relais off, LOW = Relais on)
//   ->To switch the pumps on/off
// 1 Relais Module (HIGH = Relais off, LOW = Relais on)
//   ->To switch the power to the four capacitiv humitity sensors on/off
// 4 capacitiv humidity sensors
//   ->For soil humidity measurement
// 4 100kOhm resistors
//   ->To set the analog inputs to a clean 0 at wire break
// 4 pumps
// 1 water level sensor (tank)
//   ->To avoid pump overheating due to low water level (leads to sys power off)
// 1 water level for trivet
//   ->To avoid flooding in case of arduino errors (leads to sys power off)
//   tube, sprinklers and holders
//************************************************************************************
// MOISTURE SENSOR CALIBRATION
//************************************************************************************
// Check the measurement range of all sensors.
// Once put the sensors directly in water and once leave em out of the water dry in
// the air.
// For some reason this ranges changed significantly after a few months of
// usage of the moisture sensors. Could not find out why!
// The min value is the wet value and the max value is the dry value.
//
// blue sensor 1
int blueMinValue = 258; // older values 250/306/345
int blueMaxValue = 383; // older values 379/442/571
// white sensor 2
int whiteMinValue = 231; // older values 220/280/311
int whiteMaxValue = 421; // older values 416/485/628
// brown sensor 3
int brownMinValue = 229; // older values 224/284/313
int brownMaxValue = 420; // older values 413/484/624
// grey sensor 4
int greyMinValue = 226; // older values 230/288/323
int greyMaxValue = 422; // older values 413/480/622
//************************************************************************************
// CONFIGURATION
//************************************************************************************
// LIGHT SETTINGS
// (Lights are turned on at night hours)
// lightOnHour: at this time the lights will be turned on
int lightOnHour = 15;
// lightOffHour: at this time the lights will be turned off
int lightOffHour = 9;

// HUMIDITY SETTINGS
// Turn fan on humidity
int onHumidity = 70;
// Turn fan off humidity
int offHumidity = 60;

// TEMPERATURE SETTINGS
// Turn fan on temperature
int onTemperature = 30;
// Turn fan off temperature
int offTemperature = 26;

// WATER SETTINGS
// pouringTime: once the moist sensor measures below soilMoistureDryValue the pump
//              will be turend on for this Time
//              (pouringTime in milli seconds - Set to a bigger value for bigger pots)
float pouringTime = 25000;

// soilMoistureDryValue: once the measurement of the moisture sensors is higher than
//                       this value, the pouring will be activated
//                       This will be checked once per hour
int soilMoistureDryValue = 280; // old value 305
//************************************************************************************
// LIBRARIES, DEFINES, INITIALIZATIONS
//************************************************************************************
// BASE
// Library for RTC (Real Time Clock)
#include <DS3231.h>
#include <Wire.h>
//Initialize the RTC - DS3231
DS3231 rtc;

// Library for OLED-Display
#include "SSD1306Ascii.h"
#include "SSD1306AsciiWire.h"
// I2C - Address of OLED Display
#define I2C_ADDRESS 0x3C
// Define proper RST_PIN
#define RST_PIN 4
// Create oled
SSD1306AsciiWire oled;

// HUMIDITY AND TEMPERATURE
// Library for DHT11
#include "DHT.h"
// Pin to which the DHT11 is connected
#define DHTPIN 5
// Type Definition
#define DHTTYPE DHT11 // DHT 11
DHT dht(DHTPIN, DHTTYPE);
//************************************************************************************
// Input/Output pin definitions
//************************************************************************************
// LIGHT
int lightD1 = 6;

// HUMIDITY AND TEMPERATURE
int fanD2 = 7;

// WATER
int moistureSensor1 = 0;
int moistureSensor2 = 1;
int moistureSensor3 = 2;
int moistureSensor4 = 3;

int pumpPlant1 = 12;
int pumpPlant2 = 11;
int pumpPlant3 = 10;
int pumpPlant4 = 9;

int moistureSensorPower = 8;
//************************************************************************************
// Global variables
//************************************************************************************
int actualSoilMoisture = 0;
int actualHour = 0;
bool lightIsOn = 0;
bool fanIsOn = 0;

// create variables to get actual hour from rtc object
bool h12 = false;
bool AMPM = false;
//************************************************************************************
// OLED-Display variables for all eight possible lines
//************************************************************************************
// BASE
String lineOne = "";
String lineTwo = "";
String lineThree = "";
String lineFour = "";
String lineFive = "";
String lineSix = "";
String lineSeven = "";
String lineEight = "";
//************************************************************************************
// SETUP
//************************************************************************************
void setup()
{
  //**********************************************************************************
  // GENERAL
  //**********************************************************************************
  // BASE
  // Setup serial connection
  Wire.begin();
  Serial.begin(9600);

  // Set the clock mode to 24h mode
  rtc.setClockMode(false);
  
  // The following lines can be used to set the date and time after batterychange
  /*
  // Actual hour 0 - 23
  rtc.setHour(17);
  // Actual minute 0 -59
  rtc.setMinute(40);
  // Actual second 0-59
  rtc.setSecond(0);
  // Actual day of the Month 1-31
  rtc.setDate(11);
  // Actual month 1-12
  rtc.setMonth(6);
  // Acutal year 01 - 99
  rtc.setYear(20);
  // Actual day of week
  // Be careful to set the DoW right. It has to be set!
  // Set 1 for Monday up to 7 for Sunday
  rtc.setDoW(4);
  */

  // OLED setup 
  // Wire.begin();
  Wire.setClock(400000L);
  oled.begin(&Adafruit128x64, I2C_ADDRESS, RST_PIN);
  oled.setFont(Adafruit5x7);
  
  // HUMIDITY AND TEMPERATURE
  // Initialize DHT11 sensor
  dht.begin();
  //**********************************************************************************
  // DEFINE PIN TYPES
  //**********************************************************************************
  // LIGHT
  pinMode(lightD1, OUTPUT);

  // HUMIDITY AND TEMPERATURE
  pinMode(fanD2, OUTPUT);

  // WATER
  pinMode(moistureSensorPower, OUTPUT);
  pinMode(pumpPlant1, OUTPUT);
  pinMode(pumpPlant2, OUTPUT);
  pinMode(pumpPlant3, OUTPUT);
  pinMode(pumpPlant4, OUTPUT);
  //**********************************************************************************
  // INITIALIZE PINS
  //**********************************************************************************
  // LIGHT
  digitalWrite(lightD1, HIGH);

  // HUMIDITY AND TEMPERATURE
  digitalWrite(fanD2, HIGH);

  // WATER (HIGH means that the pumps are not running)
  digitalWrite(pumpPlant1, HIGH);
  digitalWrite(pumpPlant2, HIGH);
  digitalWrite(pumpPlant3, HIGH);
  digitalWrite(pumpPlant4, HIGH);
  digitalWrite(moistureSensorPower, HIGH);
}
//************************************************************************************
// MAIN LOOP
//************************************************************************************
void loop()
{
  //**********************************************************************************
  // LIGHT
  //**********************************************************************************
  // Light is on at nighttime and off at daytime
  if ((rtc.getHour(h12,AMPM) >= lightOnHour || rtc.getHour(h12,AMPM) < lightOffHour)
       && lightIsOn == 0)
  {
    // Turn the light on
    digitalWrite(lightD1, LOW);
    lightIsOn = 1;
  }
  if ((rtc.getHour(h12,AMPM) >= lightOffHour && rtc.getHour(h12,AMPM) < lightOnHour)
       && lightIsOn == 1)
  {
    // Turn the light off
    digitalWrite(lightD1, HIGH);
    lightIsOn = 0;
  }
  //**********************************************************************************
  // HUMIDITY AND TEMPERATURE
  //**********************************************************************************
  // Reading temperature or humidity takes about 250 milliseconds!
  // Read humidity as %
  int humidity = dht.readHumidity();

  // Read temperature as Celsius
  int temperature = dht.readTemperature();

  // If onTemperature or onHumidity is exceeded the fan turns on.
  // It turns off when both offHumidity and offTemperature are undershoot.
  if (humidity > onHumidity || temperature > onTemperature && fanIsOn == 0)
  {        
    digitalWrite(fanD2, LOW);
    fanIsOn = 1;
  }
  if (humidity < offHumidity && temperature < offTemperature && fanIsOn == 1)
  {
    digitalWrite(fanD2, HIGH);
    fanIsOn = 0;
  }
  //**********************************************************************************
  // WATER
  //**********************************************************************************
  // Just a little moment of reflection - no need for this really
  // When you turn on the KS-Graden system it will wait for 5 seconds before starting
  // it's first measurement.
  delay(5000);

  // Reset actualHour at 00:00
  if (rtc.getHour(h12,AMPM) == 0)
  {
    actualHour = 0;
  }

  // Read soil moisture only once an hour (except at 0:00)
  if (actualHour < rtc.getHour(h12,AMPM))
  {
    // get actual time
    actualHour = rtc.getHour(h12,AMPM);
    // Prepare strings for OLED-Display
    String dateStr = String(rtc.getDate());
    String hourStr = String(rtc.getHour(h12,AMPM));
    String minStr = String(rtc.getMinute());
    String humidityStr = String(humidity);
    String temperatureStr = String(temperature);

    // **********************************************
    // Read soil moisture of plant 1
    digitalWrite(moistureSensorPower, LOW);
    delay(2000);
    actualSoilMoisture = analogRead(moistureSensor1);
    delay(2000);
    digitalWrite(moistureSensorPower, HIGH);
    delay(1000);
    
    // Set OLED variable
    lineOne = "1:"+dateStr+"-"+hourStr+":"+minStr+"-"+actualSoilMoisture+"   T:"
               +temperatureStr;

    // pour Plant 1
    // only pour at plausible value
    if ((actualSoilMoisture > soilMoistureDryValue-5)
         && (actualSoilMoisture < (blueMaxValue-20)))
    {
      // Run pump
      digitalWrite(pumpPlant1, LOW);
      delay(pouringTime);
      digitalWrite(pumpPlant1, HIGH);
      
      // Set OLED variable
      lineFive = "1:"+dateStr+"-"+hourStr+":"+minStr+"-"+actualSoilMoisture;
    }

    // **********************************************
    // Read soil moisture of plant 2
    digitalWrite(moistureSensorPower, LOW);
    delay(2000);
    actualSoilMoisture = analogRead(moistureSensor2);
    delay(2000);
    digitalWrite(moistureSensorPower, HIGH);
    delay(1000);

    // Set OLED variable
    lineTwo = "2:"+dateStr+"-"+hourStr+":"+minStr+"-"+actualSoilMoisture+"   H:"
               +humidityStr;

    // pour Plant 2
    // only pour at plausible value
    if ((actualSoilMoisture > soilMoistureDryValue)
         && (actualSoilMoisture < (whiteMaxValue-20)))
    {
      // Run pump
      digitalWrite(pumpPlant2, LOW);
      delay(pouringTime);
      digitalWrite(pumpPlant2, HIGH);

      // Set OLED variable
      lineSix = "2:"+dateStr+"-"+hourStr+":"+minStr+"-"+actualSoilMoisture;
    }

    // **********************************************
    // Read soil moisture of plant 3
    digitalWrite(moistureSensorPower, LOW);
    delay(2000);
    actualSoilMoisture = analogRead(moistureSensor3);
    delay(2000);
    digitalWrite(moistureSensorPower, HIGH);
    delay(1000);
    
    // Set OLED variable
    lineThree = "3:"+dateStr+"-"+hourStr+":"+minStr+"-"+actualSoilMoisture;

    // pour Plant 3
    // only pour at plausible value
    if ((actualSoilMoisture > soilMoistureDryValue)
         && (actualSoilMoisture < (brownMaxValue-20)))
    {
      // Run pump
      digitalWrite(pumpPlant3, LOW);
      delay(pouringTime);
      digitalWrite(pumpPlant3, HIGH);
      
      // Set OLED variable
      lineSeven = "3:"+dateStr+"-"+hourStr+":"+minStr+"-"+actualSoilMoisture;
    }

    // **********************************************
    // Read soil moisture of plant 4
    digitalWrite(moistureSensorPower, LOW);
    delay(2000);
    actualSoilMoisture = analogRead(moistureSensor4);
    delay(2000);
    digitalWrite(moistureSensorPower, HIGH);
    delay(1000);

    // Set OLED variable
    lineFour = "4:"+dateStr+"-"+hourStr+":"+minStr+"-"+actualSoilMoisture;

    // pour Plant 4
    // only pour at plausible value
    if ((actualSoilMoisture > soilMoistureDryValue)
         && (actualSoilMoisture < (greyMaxValue-20)))
    {
      // Run pump
      digitalWrite(pumpPlant4, LOW);
      delay(pouringTime);
      digitalWrite(pumpPlant4, HIGH);

      // Set OLED variable
      lineEight = "4:"+dateStr+"-"+hourStr+":"+minStr+"-"+actualSoilMoisture;
    }
  // Actualize the Panel
  oled.clear();
  // At the end of all measurements and pourings for all plants the display will be
  // showing the information about all measurements and pourings
  oled.println(lineOne);
  oled.println(lineTwo);
  oled.println(lineThree);
  oled.println(lineFour);
  oled.println(lineFive);
  oled.println(lineSix);
  oled.println(lineSeven);
  oled.println(lineEight);
  }
}
//************************************************************************************
